#include "TObjednavkaPokoje.h"

#include "TMyString.h"
#include "TMojeQuery.h"
//---------------------------------------------------------------------------

TObjednavkaPokoje::TObjednavkaPokoje(int _KlientID, int _CisloObjednavky,
                        const TDate & _DatumObjednavky, const TDate & _OdKdy,
                            const TDate & _DoKdy, int _PokojID, double _Cena)
    // konstruktor pro natn dat z databze
{
    KlientID = _KlientID;
    CisloObjednavky = _CisloObjednavky;
    DatumObjednavky = _DatumObjednavky;
    OdKdy = _OdKdy;
    DoKdy = _DoKdy;
    PokojID = _PokojID;
    Cena = _Cena;
    CenuNelzeDohledat = false;
}
//---------------------------------------------------------------------------

TObjednavkaPokoje::TObjednavkaPokoje(int _KlientID, const TDate & _DatumObjednavky,
    const TDate & _OdKdy, const TDate & _DoKdy, int _PokojID, int _TypPokojeID)
    // konstruktor pro novou objednavku, zbyvajici udaje si dohleda v databazi
    // pokud je _PokojID == 0, zkus dohledat voln pokoj danho typu,
    // jinak na typ kale
    // Kdy nenajde voln pokoj danho typu, piad PokojID = 0
{
    KlientID = _KlientID;
    DatumObjednavky = _DatumObjednavky;
    OdKdy = _OdKdy;
    DoKdy = _DoKdy;
    if(_PokojID)
        PokojID = _PokojID;
    else
        PokojID = NajdiVolnyPokojPodleTypu(_TypPokojeID, _OdKdy, _DoKdy);
    CisloObjednavky = 0;

    bool Chyba = false;
    Cena = NajdiCenuPokojeOdDo(_KlientID, _PokojID, _OdKdy, _DoKdy, Chyba);
    CenuNelzeDohledat = Chyba ? true : false;
}
//---------------------------------------------------------------------------

bool TObjednavkaPokoje::KolidujeSObjednavkamiVDatabazi(TMojeQuery * _Dotaz) const
{
    if(PokojID == 0) // nen piazen pokoj, je to kvuli tomu, ze jsme hledali
        return true;           // podle typu a nenasli volny, tedy koliduje

    TMojeQuery * Dotaz;
    if(!_Dotaz)
        Dotaz = new TMojeQuery();
    else
        Dotaz = _Dotaz;

    Dotaz->SQL->Clear();
    TMyString DotazStr = TMyString("SELECT COUNT(*) AS pocet FROM"
        " objednavky_pokoje_tab WHERE pokoj_id = \"") + TMyString(PokojID) +
        "\" AND (( od_kdy >= \"" + TMyString(OdKdy) + "\" AND od_kdy <= \"" +
        TMyString(DoKdy) + "\") OR ( do_kdy >= \"" + TMyString(OdKdy) +
        "\" AND do_kdy <= \"" + TMyString(DoKdy) + "\") OR "
        "( do_kdy >= \"" + TMyString(DoKdy) +
        "\" AND od_kdy <= \"" + TMyString(OdKdy) + "\"))";
        // shoduje se pokoj_id a prekryvaji se intervaly (od_kdy, do_kdy) a
        // (this->OdKdy, this->DoKdy)
    Dotaz->SQL->Add(DotazStr);
    Dotaz->Open();

    bool Vratit = false;
    Dotaz->First();
    if(Dotaz->FieldByName("pocet")->AsInteger > 0)
        Vratit = true;

    if(!_Dotaz)
        delete Dotaz;
    return Vratit;
}
//--------------------------------------------------------------------------

bool TObjednavkaPokoje::UlozDoDatabaze(TMojeQuery * _Dotaz) const
    // vrt true, pokud se uloilo
    // pokud dostane _Dotaz, pedpokld, e je tabulka zamen
{
    TMojeQuery * Dotaz;
    if(!_Dotaz)
    {
        Dotaz = new TMojeQuery();
        Dotaz->SQL->Clear();
        Dotaz->SQL->Add("LOCK TABLES objednavky_pokoje_tab WRITE");
        Dotaz->ExecSQL();
    }
    else
        Dotaz = _Dotaz;


    if(KolidujeSObjednavkamiVDatabazi(Dotaz))
        return false;

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add(VytvorDotazInsertProUlozeni());
    Dotaz->ExecSQL();

    if(!_Dotaz)
    {
        Dotaz->SQL->Clear();
        Dotaz->SQL->Add("UNLOCK TABLES");
        Dotaz->ExecSQL();
        delete Dotaz;
    }
    return true;
}
//---------------------------------------------------------------------------

TMyString TObjednavkaPokoje::VytvorDotazInsertProUlozeni(void) const
    // volano z UlozDoDatabaze()
{
    return
        TMyString("INSERT INTO objednavky_pokoje_tab SET ")
        + "klient_id = \""
        + TMyString(KlientID)
        + "\", cislo_objednavky = \""
        + TMyString(CisloObjednavky > 0 ? CisloObjednavky : NajdiDalsiCisloObjednavky())
        + "\", datum_objednavky = \""
        + TMyString(DatumObjednavky)
        + "\", od_kdy = \""
        + TMyString(OdKdy)
        + "\", do_kdy = \""
        + TMyString(DoKdy)
        + "\", pokoj_id = \""
        + TMyString(PokojID)
        + "\", cena = \""
        + TMyString(Cena)
        + "\"";
}
//---------------------------------------------------------------------------

int TObjednavkaPokoje::NajdiDalsiCisloObjednavky(void) const
    // volano z VytvorDotazInsertProUlozeni() pro novou objednavku
{
    TMojeQuery * Dotaz;
    Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("SELECT MAX(cislo_objednavky)+ 1 AS nove_cislo FROM objednavky_pokoje_tab");
    Dotaz->Open();

    Dotaz->First();
    int Vratit = Dotaz->FieldByName("nove_cislo")->AsInteger;
    delete Dotaz;
    return Vratit;
}
//---------------------------------------------------------------------------

double TObjednavkaPokoje::NajdiCenuPokojeOdDo(int _KlientID, int _PokojID,
                 const TDate & _OdKdy, const TDate & _DoKdy, bool & _Chyba) const
    // vrt celkovou cenu pokoje pro tohoto klienta, pokoj a dan obdob
{
    double Vratit = 0;
    for(TDate D = _OdKdy; (int) D <= (int) _DoKdy; D++) // (int) je kvli nepesn desetinn sti TDate
    {
        Vratit += NajdiCenuPokoje(_KlientID, _PokojID, D, _Chyba);
    }
    return Vratit;
}
//---------------------------------------------------------------------------

double TObjednavkaPokoje::NajdiCenuPokoje(int _KlientID, int _PokojID,
                                 const TDate & _Datum, bool & _Chyba) const
    // vrt cenu pokoje pro tohoto klienta, pokoj a den
{
    TMojeQuery * Dotaz;
    Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add(VytvorDotazNajdiCenuPokoje(_KlientID, _PokojID, _Datum));
    Dotaz->Open();

    double Vratit = 0;
    if(Dotaz->IsEmpty())
    {
        _Chyba = true;
    }
    else
    {
        Dotaz->First();
        Vratit = Dotaz->FieldByName("cena")->AsFloat;
    }

    delete Dotaz;
    return Vratit;
}
//---------------------------------------------------------------------------

TMyString TObjednavkaPokoje::VytvorDotazNajdiCenuPokoje(int _KlientID, int _PokojID,
                                                const TDate & _Datum) const
    // volano z NajdiCenuPokoje()
{
    return
        TMyString("SELECT cena AS cena FROM ceny_pokoje_tab, typy_dnu_tab,"
            " kalendar_typy_dnu_tab, klienti_tab, pokoje_tab WHERE")
        +   " klienti_tab.klient_id = \"" + TMyString(_KlientID) + "\""
        " AND klienti_tab.typ_klienta_id = ceny_pokoje_tab.typ_klienta_id"
        " AND pokoje_tab.pokoj_id = \"" + TMyString(_PokojID) + "\""
        " AND pokoje_tab.typ_pokoje_id = ceny_pokoje_tab.typ_pokoje_id"
        " AND kalendar_typy_dnu_tab.datum = \"" + TMyString(_Datum) + "\""
        " AND kalendar_typy_dnu_tab.typ_dne_id = ceny_pokoje_tab.typ_dne_id"
        " AND typy_dnu_tab.typ_dne_id = ceny_pokoje_tab.typ_dne_id"
        " ORDER BY typy_dnu_tab.poradi";
}
//---------------------------------------------------------------------------

int TObjednavkaPokoje::NajdiVolnyPokojPodleTypu(int _TypPokojeID,
                          const TDate & _OdKdy, const TDate & _DoKdy) const
    // K _TypPokojeID se pokus najt voln pokoj pro toto obdob a vrt
    // PokojID, pokud nenajde voln pokoj, vrt 0
    // pokud je vc volnch, vezme jeden z nich
{
    TMojeQuery * Dotaz;
    Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add(VytvorDotazNajdiVolnyPokojPodleTypu(_TypPokojeID,
                        TMyString(_OdKdy), TMyString(_DoKdy)));
    Dotaz->Open();

    int Vratit = 0;
    if(!Dotaz->IsEmpty())
    {
        Dotaz->First();
        Vratit = Dotaz->FieldByName("pokoj_id")->AsInteger;
    }

    delete Dotaz;
    return Vratit;
}
//---------------------------------------------------------------------------

TMyString TObjednavkaPokoje::VytvorDotazNajdiVolnyPokojPodleTypu(
    int _TypPokojeID, const TMyString & _OdKdy, const TMyString & _DoKdy) const
    // volano z NajdiVolnyPokojPodleTypu()
{
    return
        TMyString("SELECT pokoje.pokoj_id FROM pokoje_tab AS pokoje"
        " LEFT JOIN objednavky_pokoje_tab AS objednavky ON"
        " (pokoje.pokoj_id = objednavky.pokoj_id AND"
        " (( objednavky.od_kdy >= \"") + _OdKdy + "\" AND"
        " objednavky.od_kdy <= \"" + _DoKdy + "\") OR"
        " ( objednavky.do_kdy >= \"" + _OdKdy +
        "\" AND objednavky.do_kdy <= \"" + _DoKdy + "\") OR"
        " ( objednavky.od_kdy <= \"" + _OdKdy +
        "\" AND objednavky.do_kdy >= \"" + _DoKdy + "\")))"
        " WHERE pokoje.typ_pokoje_id = \"" + TMyString(_TypPokojeID) + "\""
        " AND objednavky.cislo_objednavky IS NULL GROUP BY pokoje.pokoj_id";
}
//---------------------------------------------------------------------------

