#include "TSeznamSkupinPokoju.h"

#include "TMojeQuery.h"

//---------------------------------------------------------------------------

TSeznamSkupinPokoju::TSeznamSkupinPokoju(void)
{
    TMojeQuery * DotazSkupinyPokoju = new TMojeQuery();

    DotazSkupinyPokoju->SQL->Add("SELECT * FROM skupiny_pokoju_tab");
    DotazSkupinyPokoju->Open();

    SkupinyPokoju.clear();

    if(!DotazSkupinyPokoju->IsEmpty())
    {
        DotazSkupinyPokoju->First();
        do
        {
            SkupinyPokoju.push_back(TSkupinaPokoju(
                DotazSkupinyPokoju->FieldByName("skupina_pokoju_id")->AsInteger,
                TMyString(DotazSkupinyPokoju->FieldByName("nazev_skupiny_pokoju")->AsString)
                                                                                    ));
            DotazSkupinyPokoju->Next();
        }
        while(!DotazSkupinyPokoju->Eof);
    }

    delete DotazSkupinyPokoju;
}
//---------------------------------------------------------------------------

void TSeznamSkupinPokoju::GetSeznamTStringList(TStrings * _sl) const
    // do seznamu TStrings predaneho pres pointer vyplni nazvy skupin pokoju,
    // co tam bylo predtim smaze
{
    vector< TSkupinaPokoju >::const_iterator it;

    _sl->Clear();

    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        _sl->Add(it->GetNazevSkupinyPokoju());
    }
}
//--------------------------------------------------------------------------

void TSeznamSkupinPokoju::GetSeznamTStringListPokojuVeSkupine(
                const TMyString & _NazevSkupinyPokoju, TStrings * _sl) const
    // do seznamu TStrings predaneho pres pointer vyplni nazvy pokoju ktere
    // jsou ve skupin _NazevSkupinyPokoju
    // co tam bylo predtim smaze
{
    vector< TSkupinaPokoju >::const_iterator it;

    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        if(it->GetNazevSkupinyPokoju() == _NazevSkupinyPokoju)
            it->GetSeznamTStringList(_sl);
    }
}
//--------------------------------------------------------------------------

int TSeznamSkupinPokoju::GetSkupinaPokojuID(
                                   const TMyString & _NazevSkupinyPokoju) const
    // podle nazvu skupiny pokoju dohleda ID, predpoklada, ze nazev je spravny
{
    vector< TSkupinaPokoju >::const_iterator it;
    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        if(it->GetNazevSkupinyPokoju() == _NazevSkupinyPokoju)
            return it->GetSkupinaPokojuID();
    }

    return -1; //sem by to nemelo dojit
}
//----------------------------------------------------------------------------

void TSeznamSkupinPokoju::OdstranSkupinuPokoju(const TMyString & _NazevSkupinyPokoju)
    // pokud je _NazevSkupinyPokoju patn, nic nedl
    // volano z "TAdministraceSkupinyPokoju"
{
    vector< TSkupinaPokoju >::iterator it;
    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        if(it->GetNazevSkupinyPokoju() == _NazevSkupinyPokoju)
        {
            SkupinyPokoju.erase(it);
            break; // mela by tam byt jenom jedna, nema smysl pokracovat
        }
    }
}
//----------------------------------------------------------------------------

void TSeznamSkupinPokoju::PridejSkupinuPokoju(const TMyString & _NazevSkupinyPokoju)
    // Prida skupinu pokoju, volano z "TAdministraceSkupinyPokoju"
{
    SkupinyPokoju.push_back(TSkupinaPokoju(NajdiNepouziteID(), _NazevSkupinyPokoju));
}
//-----------------------------------------------------------------------

int TSeznamSkupinPokoju::NajdiNepouziteID(void) const
    // Vyuziva ji "PridejSkupinuPokoju()", vrati prvni ID skupiny pokoju, ktere
    // neni ve vectoru SkupinyPokoju. To, ze ho mel nekdy drive jiny typ
    // neni na zavadu
{
    for(int NoveID =1; ; NoveID++) // for bez podminky, nekdy musi skoncit na
                                   // return v tele
    {
        if(!SeznamSkupinPokojuObsahujeID(NoveID))
        {
            return NoveID;
        }
    }
}
//-------------------------------------------------------------------------

bool TSeznamSkupinPokoju::SeznamSkupinPokojuObsahujeID(int _ID) const
    // Vyuziva ji "NajdiNepouziteID()", pokud vector "SkupinyPokoju" obsahuje
    // skupinu s danym ID, vrati true
{
    vector< TSkupinaPokoju >::const_iterator it;
    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        if(it->GetSkupinaPokojuID() == _ID)
            return true;
    }

    return false; //v celem seznamu to nenaslo _ID
}
//--------------------------------------------------------------------------

void TSeznamSkupinPokoju::UlozDoDatabaze(void) const
    // ulozi udaje z vektoru SkupinyPokoju do databaze
    // smae poloky z tabulky skupiny_pokoju_propojeni_tab, kter obsahuj ID
    // typ skupin, je byly odstranny
    // chtelo by vylepsit, pokud by pri ukladani nastala chyba, jsou data
    // ztracena
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("LOCK TABLES skupiny_pokoju_tab WRITE, skupiny_pokoju_propojeni_tab WRITE");
    Dotaz->ExecSQL();

    UlozDoDatabazeSkupinyPokoju();
    UlozDoDatabazeSkupinyPokojuPropojeni();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("UNLOCK TABLES");
    Dotaz->ExecSQL();

    delete Dotaz;
}
//---------------------------------------------------------------------------

void TSeznamSkupinPokoju::UlozDoDatabazeSkupinyPokoju(void) const
    // vyuv ji UlozDoDatabaze()
    // aktualizuje tabulku skupiny_pokoju_tab
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("DELETE FROM skupiny_pokoju_tab");
    Dotaz->ExecSQL();

    vector< TSkupinaPokoju >::const_iterator it;
    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        Dotaz->SQL->Clear();
        Dotaz->SQL->Add(VytvorDotazInsertProUlozeniSkupinyPokoju(it));
        Dotaz->ExecSQL();
    }

    delete Dotaz;
}
//---------------------------------------------------------------------------

void TSeznamSkupinPokoju::UlozDoDatabazeSkupinyPokojuPropojeni(void) const
    // vyuv ji UlozDoDatabaze()
    // aktualizuje tabulku skupiny_pokoju_propojeni_tab
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("DELETE FROM skupiny_pokoju_propojeni_tab");
        // i kdy funkce ZapisDoTabulkySkupinyPokojuPropojeni() maze radky,
        // musm toto udlat kvli smazn zznam o skupinch, kter u
        // v seznamu nejsou
    Dotaz->ExecSQL();

    vector< TSkupinaPokoju >::const_iterator it;
    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        it->ZapisDoTabulkySkupinyPokojuPropojeni();
    }

    delete Dotaz;
}
//---------------------------------------------------------------------------

TMyString TSeznamSkupinPokoju::VytvorDotazInsertProUlozeniSkupinyPokoju(
                      const vector< TSkupinaPokoju >::const_iterator _it) const
    // Vyuv ji "UlozDoDatabazeSkupinyPokoju()"
{
    return
        TMyString("INSERT INTO skupiny_pokoju_tab SET ")
        + "skupina_pokoju_id = \""
        + TMyString(_it->GetSkupinaPokojuID())
        + "\", nazev_skupiny_pokoju = \""
        + _it->GetNazevSkupinyPokoju()
        + "\"";
}
//------------------------------------------------------------------------

void TSeznamSkupinPokoju::OdstranPokojeZeSkupiny(const TMyString & _NazevSkupinyPokoju,
                                 const vector< TPokoj > & _PokojeKOdstraneni)
    // ve skupine _NazevSkupiny odstrani vsechny pokoje z vektoru _PokojeKOdstraneni
    // pokoje poznv podle PokojID
    // pokud pokoj ve skupine neni nic se nedeje
{
    vector< TSkupinaPokoju >::iterator it;
    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        if(it->GetNazevSkupinyPokoju() == _NazevSkupinyPokoju)
        {
            it->OdstranPokojeZeSkupiny(_PokojeKOdstraneni);
        }
    }
}
//---------------------------------------------------------------------------

void TSeznamSkupinPokoju::PridejPokojeDoSkupiny(const TMyString & _NazevSkupinyPokoju,
                                   const vector< TPokoj > & _PokojeNaPridani)
    // do skupiny _NazevSkupiny pid vechny pokoje z vektoru _PokojeNaPridani
    // kdy tam pokoj u je nevad
{
    vector< TSkupinaPokoju >::iterator it;
    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        if(it->GetNazevSkupinyPokoju() == _NazevSkupinyPokoju)
        {
            it->PridejPokojeDoSkupiny(_PokojeNaPridani);
        }
    }
}
//---------------------------------------------------------------------------

void TSeznamSkupinPokoju::ZobrazObsazenostPokojuProSkupinu(
    const TMyString & _NazevSkupinyPokoju, const TControl * _Parent, int _lhrX,
        int _lhrY, const TDate & _Od, const TDate & _Do,
        TSeznamVytvorenychKomponent * _SeznamVytvorenychKomponent) const
    // na komponentu _Parent od pozice _lhrX a _lhrY vype tabulku obsazenosti
    // pokoj ve skupin _NazevSkupinyPokoju od data _Od do _Do, komponenty,
    // kter k tomu vytvo ulo do _SeznamVytvorenychKomponent
{
    vector< TSkupinaPokoju >::const_iterator it;
    for(it = SkupinyPokoju.begin(); it != SkupinyPokoju.end(); it++)
    {
        if(it->GetNazevSkupinyPokoju() == _NazevSkupinyPokoju)
            it->ZobrazObsazenostPokoju(_Parent, _lhrX, _lhrY, _Od, _Do,
                                                _SeznamVytvorenychKomponent);
    }
}
//---------------------------------------------------------------------------
