#include "TSeznamTypuDni.h"

#include "TMojeQuery.h"

//--------------------------------------------------------------------

TSeznamTypuDni::TSeznamTypuDni(void)
{
    TMojeQuery * DotazTypyDni = new TMojeQuery();

    DotazTypyDni->SQL->Add(VytvorDotazTypyDniSelect());
    DotazTypyDni->Open();

    TypyDni.clear();

    if(!DotazTypyDni->IsEmpty())
    {
        DotazTypyDni->First();
        do
        {
            TypyDni.push_back(vector< TTypDne >::value_type(
                DotazTypyDni->FieldByName("typ_dne_id")->AsInteger,
                TMyString(DotazTypyDni->FieldByName("nazev_typu_dne")->AsString)
                                                                                    ));
            DotazTypyDni->Next();
        }
        while(!DotazTypyDni->Eof);
    }

    delete DotazTypyDni;

    Pozice = TypyDni.begin();
}
//-------------------------------------------------------------------

TMyString TSeznamTypuDni::VytvorDotazTypyDniSelect(void) const
    // Vyuv ji konstruktor
{
    return TMyString("SELECT * FROM typy_dnu_tab");
}
//---------------------------------------------------------------------

void TSeznamTypuDni::GetSeznamTStringList(TStrings * _sl) const
    // do seznamu TStrings pedanho pes pointer vypln nzvy typ dn
    // v seznamu, co tam bylo pedtm smae
{
    vector< TTypDne >::const_iterator it;

    _sl->Clear();

    for(it = TypyDni.begin(); it != TypyDni.end(); it++)
    {
        _sl->Add(it->GetNazevTypuDne());
    }
}
//-------------------------------------------------------------

int TSeznamTypuDni::NajdiNepouziteID(void) const
    // Vyuv ji PridejTypDne(),
    // vrt prvn slo, co nen v seznamu pouito jako ID typu dne
{
    for(int NoveID =1; ; NoveID++) // for bez podmnky je OK
    {
        if(!SeznamObsahujeID(NoveID))
        {
            return NoveID;
        }
    }
}
//-------------------------------------------------------------------------

bool TSeznamTypuDni::SeznamObsahujeID(int _ID) const
    // vyuv ji "NajdiNepouziteID()", pokud seznam "TypyDni" obsahuje _ID,
    // vrac true
{
    vector< TTypDne >::const_iterator it;
    for(it = TypyDni.begin(); it != TypyDni.end(); it++)
    {
        if(it->GetTypDneID() == _ID)
            return true;
    }

    return false; //v celem seznamu to nenaslo _ID
}
//--------------------------------------------------------------------------

void TSeznamTypuDni::PridejTypDne(const TMyString & _NazevTypuDne)
{
    TypyDni.push_back(TTypDne(NajdiNepouziteID(), _NazevTypuDne));
}
//---------------------------------------------------------------------------

void TSeznamTypuDni::OdstranTypDne(const TMyString & _NazevTypuDne)
    // pokud lze odstranit typ dne (uz k nemu nejsou zadna data v
    // "kalendar_typy_dnu"), odstrani ho a vrati true, jinak false
{
    int TypID = GetTypDneID(_NazevTypuDne);

    vector< TTypDne >::iterator it;
    for(it = TypyDni.begin(); it != TypyDni.end(); it++)
    {
        if(it->GetTypDneID() == TypID)
        {
            TypyDni.erase(it);
            break; // mel by tam byt jenom jeden, nema smysl pokracovat
        }
    }
}
//--------------------------------------------------------------------------

int TSeznamTypuDni::GetTypDneID(const TMyString & _TypDne) const
    // k nzvu typu dne najde pslun ID
{
    vector< TTypDne >::const_iterator it;
    for(it = TypyDni.begin(); it != TypyDni.end(); it++)
    {
        if(it->GetNazevTypuDne() == _TypDne)
            return it->GetTypDneID();
    }

    return -1; //sem by to nemelo dojit
}
//--------------------------------------------------------------------

void TSeznamTypuDni::UlozDoDatabaze(void) const
    // ulo "TypyDni" do databze, chtlo by vylepit, pi chyb pi ukldn
    // me dojt ke ztrt dat
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("LOCK TABLES typy_dnu_tab WRITE, kalendar_typy_dnu_tab WRITE"
                                            ", ceny_pokoje_tab WRITE");
    Dotaz->ExecSQL();

    UlozDoDatabazeTypyDni();
    UlozDoDatabazeOdstranCenyPokoje();
    UlozDoDatabazeOdstranKalendarTypyDnu();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("UNLOCK TABLES");
    Dotaz->ExecSQL();

    delete Dotaz;
}
//-----------------------------------------------------------------------

void TSeznamTypuDni::UlozDoDatabazeTypyDni(void) const
    // vyuv ji UlozDoDatabaze()
    // aktualizuje tabulku typy_pokoju_tab
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("DELETE FROM typy_dnu_tab");
    Dotaz->ExecSQL();

    vector< TTypDne >::const_iterator it;
    for(it = TypyDni.begin(); it != TypyDni.end(); it++)
    {
        Dotaz->SQL->Clear();
        Dotaz->SQL->Add(VytvorDotazInsertProUlozeniTypuDni(it));
        Dotaz->ExecSQL();
    }

    delete Dotaz;
}
//---------------------------------------------------------------------------

void TSeznamTypuDni::UlozDoDatabazeOdstranCenyPokoje(void) const
    // vyuv ji UlozDoDatabaze()
    // aktualizuje tabulku ceny_pokoje_tab
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    TMyString DotazStr = "DELETE FROM ceny_pokoje_tab WHERE 1";

    vector< TTypDne >::const_iterator it;
    for(it = TypyDni.begin(); it != TypyDni.end(); it++)
    {
        DotazStr += " AND";
        DotazStr += TMyString(" typ_dne_id <> \"") + TMyString(it->GetTypDneID()) + "\"";
    }
    Dotaz->SQL->Add(DotazStr);
    Dotaz->ExecSQL();
    delete Dotaz;
}
//---------------------------------------------------------------------------

void TSeznamTypuDni::UlozDoDatabazeOdstranKalendarTypyDnu(void) const
    // vyuv ji UlozDoDatabaze()
    // aktualizuje tabulku kalendar_typy_dnu_tab
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    TMyString DotazStr = "DELETE FROM kalendar_typy_dnu_tab WHERE 1";

    vector< TTypDne >::const_iterator it;
    for(it = TypyDni.begin(); it != TypyDni.end(); it++)
    {
        DotazStr += " AND";
        DotazStr += TMyString(" typ_dne_id <> \"") + TMyString(it->GetTypDneID()) + "\"";
    }
    Dotaz->SQL->Add(DotazStr);
    Dotaz->ExecSQL();
    delete Dotaz;
}
//---------------------------------------------------------------------------

TMyString TSeznamTypuDni::VytvorDotazInsertProUlozeniTypuDni(
                            const vector< TTypDne >::const_iterator _it) const
    // vyuv ji "UlozDoDatabaze()"
{
    return
        TMyString("INSERT INTO typy_dnu_tab SET ")
        + "typ_dne_id = \""
        + TMyString(_it->GetTypDneID())
        + "\", nazev_typu_dne = \""
        + _it->GetNazevTypuDne()
        + "\"";
}
//------------------------------------------------------------------------

bool TSeznamTypuDni::First(void)
    // pokud v seznamu nco je, nastav intern ukazatel na prvn prvek a
    // vrt true
{
    Pozice = TypyDni.begin();
    if(Pozice != TypyDni.end())
    {
        return true;
    }
    else
    {
        return false;
    }
}
//----------------------------------------------

bool TSeznamTypuDni::Next(void)
    // pokud intern ukazatel nen na poslednm prvku, posune ho a vrt true
{
    if(Pozice && ((Pozice + 1) != TypyDni.end()))
    {
        Pozice++;
        return true;
    }
    else
    {
        return false;
    }
}
//-----------------------------------------------------

int TSeznamTypuDni::PoziceGetTypDneID(void) const
    // vrt ID typu dne, na kter ukazuje intern ukazatel
{
    return Pozice->GetTypDneID();
}
//----------------------------------------------------------

