#include "TSeznamTypuPokoju.h"

#include "TMojeQuery.h"

//---------------------------------------------------

TSeznamTypuPokoju::TSeznamTypuPokoju(void)
{
    TMojeQuery * DotazTypyPokoju = new TMojeQuery();

    DotazTypyPokoju->SQL->Add(VytvorDotazTypyPokojuSelect());
    DotazTypyPokoju->Open();

    TypyPokoju.clear();

    if(!DotazTypyPokoju->IsEmpty())
    {
        DotazTypyPokoju->First();
        do
        {
            TypyPokoju.push_back(vector< TTypPokoje >::value_type(
                DotazTypyPokoju->FieldByName("typ_pokoje_id")->AsInteger,
                TMyString(DotazTypyPokoju->FieldByName("nazev_typu_pokoje")->AsString),
                DotazTypyPokoju->FieldByName("pocet_luzek")->AsInteger
                                                                                    ));
            DotazTypyPokoju->Next();
        }
        while(!DotazTypyPokoju->Eof);
    }

    delete DotazTypyPokoju;
}

//----------------------------------------------------

TMyString TSeznamTypuPokoju::VytvorDotazTypyPokojuSelect(void) const
    // Vyuv ji konstruktor
{
    return "SELECT * FROM typy_pokoju_tab";
}
//------------------------------------------------------

void TSeznamTypuPokoju::GetSeznamTStringList(TStrings * _sl) const
    // do seznamu TStrings predaneho pres pointer vyplni nazvy typu pokoju,
    // co tam bylo predtim smaze
{
    vector< TTypPokoje >::const_iterator it;

    _sl->Clear();

    for(it = TypyPokoju.begin(); it != TypyPokoju.end(); it++)
    {
        _sl->Add(it->GetNazevTypuPokoje());
    }
}
//-------------------------------------------------------------

int TSeznamTypuPokoju::GetTypPokojeID(const TMyString & _TypPokoje) const
    // podle nazvu typu pokoje dohleda ID, predpoklada, ze nazev je spravny
{
    vector< TTypPokoje >::const_iterator it;
    for(it = TypyPokoju.begin(); it != TypyPokoju.end(); it++)
    {
        if(it->GetNazevTypuPokoje() == _TypPokoje)
            return it->GetTypPokojeID();
    }

    return -1; //sem by to nemelo dojit
}
//--------------------------------------------------------------------

void TSeznamTypuPokoju::UlozDoDatabaze(void) const
    // ulozi udaje z vektoru do databaze
    // smae poloky z tabulky ceny_pokoje_tab, kter obsahuj ID typ pokoj,
    // je byly odstranny
    // chtelo by vylepsit, pokud by pri ukladani nastala chyba, jsou data
    // ztracena
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("LOCK TABLES typy_pokoju_tab WRITE, ceny_pokoje_tab WRITE");
    Dotaz->ExecSQL();

    UlozDoDatabazeTypyPokoju();
    UlozDoDatabazeOdstranCenyPokoje();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("UNLOCK TABLES");
    Dotaz->ExecSQL();

    delete Dotaz;
}
//-----------------------------------------------------------------------

void TSeznamTypuPokoju::UlozDoDatabazeTypyPokoju(void) const
    // vyuv ji UlozDoDatabaze()
    // aktualizuje tabulku typy_pokoju_tab
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add("DELETE FROM typy_pokoju_tab");
    Dotaz->ExecSQL();

    vector< TTypPokoje >::const_iterator it;
    for(it = TypyPokoju.begin(); it != TypyPokoju.end(); it++)
    {
        Dotaz->SQL->Clear();
        Dotaz->SQL->Add(VytvorDotazInsertProUlozeni(it));
        Dotaz->ExecSQL();
    }

    delete Dotaz;
}
//---------------------------------------------------------------------------

void TSeznamTypuPokoju::UlozDoDatabazeOdstranCenyPokoje() const
    // vyuv ji UlozDoDatabaze()
    // aktualizuje tabulku ceny_pokoje_tab
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    TMyString DotazStr = "DELETE FROM ceny_pokoje_tab WHERE 1";

    vector< TTypPokoje >::const_iterator it;
    for(it = TypyPokoju.begin(); it != TypyPokoju.end(); it++)
    {
        DotazStr += " AND";
        DotazStr += TMyString(" typ_pokoje_id <> \"") + TMyString(it->GetTypPokojeID()) + "\"";
    }
    Dotaz->SQL->Add(DotazStr);
    Dotaz->ExecSQL();
    delete Dotaz;
}
//---------------------------------------------------------------------------

void TSeznamTypuPokoju::PridejTypPokoje(const TMyString & _NazevTypu,
                                                            int _PocetLuzek)
    // Prida typ pokoje, volano z "TAdministraceTypyPokoju"
{
    TypyPokoju.push_back(TTypPokoje(NajdiNepouziteID(), _NazevTypu, _PocetLuzek));
}
//-----------------------------------------------------------------------

bool TSeznamTypuPokoju::OdstranTypPokoje(const TMyString & _NazevTypu)
    // Vrt false pokud typ neme bt odstrann, protoe tabulka "pokoje_tab"
    // obsahuje jet njak pokoje tohoto typu. Jinak vrac true a odstran ho
    // z vectoru "TypyPokoju"
{
    int TypID = GetTypPokojeID(_NazevTypu);
    if(!TabulkaPokojuObsahujeIDTypuPokoje(TypID))
    {
        OdstranTypPokojeBezKontroly(TypID);
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------

void TSeznamTypuPokoju::OdstranTypPokojeBezKontroly(int _TypID)
    // Vyuv ji metoda OdstranTypPokoje(), tato metoda by nemla bt volna
    // pmo, protoe hroz poruen integrity dat v databzi - v tabulce
    // "pokoje_tab" budou pokoje neexistujcho typu
{
    vector< TTypPokoje >::iterator it;
    for(it = TypyPokoju.begin(); it != TypyPokoju.end(); it++)
    {
        if(it->GetTypPokojeID() == _TypID)
        {
            TypyPokoju.erase(it);
            break; // mel by tam byt jenom jeden, nema smysl pokracovat
        }
    }
}
//--------------------------------------------------------------------------

bool TSeznamTypuPokoju::TabulkaPokojuObsahujeIDTypuPokoje(int _ID) const
    // Vyuv ji OdstranTypPokoje() ke kontrole, zda se jet v tabulce
    // "pokoje_tab" vyskytuj njak pokoje typu _ID. Pokud ano, vrac true.
{
    bool Obsahuje = false;
    TMojeQuery * PokojeKontrolaQuery = new TMojeQuery();
    PokojeKontrolaQuery->SQL->Clear();
    PokojeKontrolaQuery->SQL->Add(TMyString(
            "SELECT typ_pokoje_id FROM pokoje_tab WHERE typ_pokoje_id = \""
                                                    + TMyString(_ID) + "\""));
    PokojeKontrolaQuery->Open();
    if(PokojeKontrolaQuery->RecordCount > 0)
    {
        Obsahuje = true;
    }

    delete PokojeKontrolaQuery;
    return Obsahuje;
}
//--------------------------------------------------------------------------

TMyString TSeznamTypuPokoju::VytvorDotazInsertProUlozeni(
                        const vector< TTypPokoje >::const_iterator _it) const
    // Vyuv ji "UlozDoDatabazeTypyPokoju()"
{
    return
        TMyString("INSERT INTO typy_pokoju_tab SET ")
        + "typ_pokoje_id = \""
        + TMyString(_it->GetTypPokojeID())
        + "\", nazev_typu_pokoje = \""
        + _it->GetNazevTypuPokoje()
        + "\", pocet_luzek = \""
        + TMyString(_it->GetPocetLuzek())
        + "\"";
}
//------------------------------------------------------------------------

int TSeznamTypuPokoju::NajdiNepouziteID(void) const
    // Vyuziva ji "PridejTypPokoje()", vrati prvni ID typu pokoje, ktere
    // neni ve vectoru TypyPokoju. To, ze ho mel nekdy drive jiny typ
    // neni na zavadu
{
    for(int NoveID =1; ; NoveID++) // for bez podminky, nekdy musi skoncit na
                                   // return v tele
    {
        if(!SeznamObsahujeID(NoveID))
        {
            return NoveID;
        }
    }
}
//-------------------------------------------------------------------------

bool TSeznamTypuPokoju::SeznamObsahujeID(int _ID) const
    // Vyuziva ji "NajdiNepouziteID()", pokud vector "TypyPokoju" obsahuje
    // pokoj s danym ID, vrati true
{
    vector< TTypPokoje >::const_iterator it;
    for(it = TypyPokoju.begin(); it != TypyPokoju.end(); it++)
    {
        if(it->GetTypPokojeID() == _ID)
            return true;
    }

    return false; //v celem seznamu to nenaslo _ID
}
//--------------------------------------------------------------------------

