#include "TSkupinaPokoju.h"

#include "TMojeQuery.h"

#include <algorithm>

using std::find;
//---------------------------------------------------------------------------

TSkupinaPokoju::TSkupinaPokoju(int _SkupinaPokojuID,
                                        const TMyString & _NazevSkupinyPokoju)
{
    SkupinaPokojuID = _SkupinaPokojuID;
    NazevSkupinyPokoju = _NazevSkupinyPokoju;

    NaplnVektorZDatabaze(SkupinaPokojuID);
}
//---------------------------------------------------------------------------

void TSkupinaPokoju::NaplnVektorZDatabaze(int _SkupinaPokojuID)
    // volno z konstruktoru, nat data z databze
{
    TMojeQuery * DotazPokoje = new TMojeQuery();

    DotazPokoje->SQL->Add(TMyString("SELECT"
        " pokoje_tab.pokoj_id, pokoje_tab.cislo_pokoje, pokoje_tab.typ_pokoje_id"
        " FROM skupiny_pokoju_propojeni_tab, pokoje_tab WHERE"
        " skupiny_pokoju_propojeni_tab.pokoj_id = pokoje_tab.pokoj_id AND"
        " skupina_pokoju_id = \"") + TMyString(_SkupinaPokojuID) + "\"");
    DotazPokoje->Open();

    SeznamPokojuVeSkupine.clear();

    if(!DotazPokoje->IsEmpty())
    {
        DotazPokoje->First();
        do
        {
            SeznamPokojuVeSkupine.push_back(TPokoj(
                DotazPokoje->FieldByName("pokoj_id")->AsInteger,
                TMyString(DotazPokoje->FieldByName("cislo_pokoje")->AsString),
                DotazPokoje->FieldByName("typ_pokoje_id")->AsInteger
                                                                          ));
            DotazPokoje->Next();
        }
        while(!DotazPokoje->Eof);
    }

    delete DotazPokoje;
}
//---------------------------------------------------------------------------

void TSkupinaPokoju::GetSeznamTStringList(TStrings * _sl) const
    // do pedanho pointeru ukazujcho na TStrings vypln
    // sla pokoj ze skupiny, ve co tam bylo pedtm odstran
{
    vector< TPokoj >::const_iterator it;

    _sl->Clear();

    for(it = SeznamPokojuVeSkupine.begin(); it != SeznamPokojuVeSkupine.end(); it++)
    {
        _sl->Add(it->GetCisloPokoje());
    }
}
//----------------------------------------------------------------------------

void TSkupinaPokoju::OdstranPokojeZeSkupiny(
                                 const vector< TPokoj > & _PokojeKOdstraneni)
    // vechny pokoje, kter obsahuje vektor _PokojeKOdstraneni kter se
    // nachzej v tto skupin odstran, kdy tu nkter nen, nevad
{
    vector< TPokoj >::const_iterator it;
    vector< TPokoj >::iterator PokojKeSmazani;
    for(it = _PokojeKOdstraneni.begin(); it != _PokojeKOdstraneni.end(); it++)
    {
        if((PokojKeSmazani = find(SeznamPokojuVeSkupine.begin(), SeznamPokojuVeSkupine.end(),
                *it)) != SeznamPokojuVeSkupine.end()) //
        {
            SeznamPokojuVeSkupine.erase(PokojKeSmazani);
        }
    }
}
//---------------------------------------------------------------------------

void TSkupinaPokoju::PridejPokojeDoSkupiny(
                                   const vector< TPokoj > & _PokojeNaPridani)
    // pid do skupiny vechny pokoje z vektoru _PokojeNaPridani
    // duplicitn nevad, nepidaj se
{
    vector< TPokoj >::const_iterator it;
    for(it = _PokojeNaPridani.begin(); it != _PokojeNaPridani.end(); it++)
    {
        if(find(SeznamPokojuVeSkupine.begin(), SeznamPokojuVeSkupine.end(),
                *it) == SeznamPokojuVeSkupine.end()) // pokoj tam jeste neni
        {
            SeznamPokojuVeSkupine.push_back(*it);
        }
    }
}
//---------------------------------------------------------------------------

void TSkupinaPokoju::ZapisDoTabulkySkupinyPokojuPropojeni(void) const
    // aktualizuje v tabulce "skupiny_pokoju_propojeni_tab" zznamy patc
    // tto skupin pokoj, smae co tu nejsou a vlo nov
{
    TMojeQuery * Dotaz = new TMojeQuery();

    Dotaz->SQL->Clear();
    Dotaz->SQL->Add(TMyString("DELETE FROM skupiny_pokoju_propojeni_tab WHERE"
        " skupina_pokoju_id = \"") + TMyString(SkupinaPokojuID) + "\"");
    Dotaz->ExecSQL();

    vector< TPokoj >::const_iterator it;
    for(it = SeznamPokojuVeSkupine.begin(); it != SeznamPokojuVeSkupine.end(); it++)
    {
        Dotaz->SQL->Clear();
        Dotaz->SQL->Add(VytvorDotazInsertProZapisDoTabulkySkupinyPokojuPropojeni(it));
        Dotaz->ExecSQL();
    }

    delete Dotaz;
}
//---------------------------------------------------------------------------

TMyString TSkupinaPokoju::VytvorDotazInsertProZapisDoTabulkySkupinyPokojuPropojeni(
                            const vector< TPokoj >::const_iterator _it) const
    // volno z ZapisDoTabulkySkupinyPokojuPropoje()
{
    return
        TMyString("INSERT INTO skupiny_pokoju_propojeni_tab SET ")
        + "skupina_pokoju_id = \""
        + TMyString(SkupinaPokojuID)
        + "\", pokoj_id = \""
        + TMyString(_it->GetPokojID())
        + "\"";
}
//----------------------------------------------------------------------------

void TSkupinaPokoju::ZobrazObsazenostPokoju(const TControl * _Parent,
                int _lhrX, int _lhrY, const TDate & _Od, const TDate & _Do,
                    TSeznamVytvorenychKomponent * _SeznamVytvorenychKomponent) const
    // na komponentu _Parent od pozice _lhrX a _lhrY vype tabulku obsazenosti
    // pokoj od data _Od do _Do, komponenty,
    // kter k tomu vytvo ulo do _SeznamVytvorenychKomponent
{
    _lhrY = 2 + ZobrazDataNadCtverecky(_Parent, _lhrX + ObsazenostSirkaNazvuPokoju,
             _lhrY, ObsazenostPosunutiX, _Od, _Do, _SeznamVytvorenychKomponent);

    vector< TPokoj >::const_iterator it;
    for(it = SeznamPokojuVeSkupine.begin(); it != SeznamPokojuVeSkupine.end(); it++)
    {
        it->ZobrazLabelCisloPokoje(_Parent, _lhrX, _lhrY,
                                                _SeznamVytvorenychKomponent);
        it->ZobrazObsazenostPokojeOdDo(_Parent, _lhrX + ObsazenostSirkaNazvuPokoju,
             _lhrY, ObsazenostPosunutiX, _Od, _Do, _SeznamVytvorenychKomponent);

        _lhrY += ObsazenostPosunutiY;
    }
}
//----------------------------------------------------------------------------

int TSkupinaPokoju::ZobrazDataNadCtverecky(const TControl * _Parent, int _lhrX,
            int _lhrY, int _MezeryX, const TDate & _Od, const TDate & _Do,
            TSeznamVytvorenychKomponent * _SeznamVytvorenychKomponent) const
    // od souadnic _lhrX, _lhrY na formul _Parent vype data a dny v tdnu
    // vytvoen komponenty pid do _SeznamVytvorenychKomponent
    // vrt spodek komponent co vykreslil
{
    int lhrY = _lhrY;
    for(TDate D = _Od; (int) D <= (int) _Do; D++) // mus bt (int), desetinn
            // st se toti nemus shodovat
    {
        _lhrY = ZobrazDatumNadCtvereckem(_Parent, _lhrX, lhrY, D,
                    _SeznamVytvorenychKomponent);
        _lhrX += _MezeryX;
    }
    return _lhrY;
}
//----------------------------------------------------------------------------

int TSkupinaPokoju::ZobrazDatumNadCtvereckem(const TControl * _Parent,
            int _lhrX, int _lhrY, const TDate & _Datum,
            TSeznamVytvorenychKomponent * _SeznamVytvorenychKomponent) const
    // volno ze ZobrazDataNadCtverecky()
    // vrt spodek komponent co vykreslil
    // pouv komponenty TLabel z pedanho pole od indexu _PoleLabelIndex a ten
    // pslun zvt
{
    TDate D = _Datum;
    TLabel * DenVTydnu = new TLabel(0);
    _SeznamVytvorenychKomponent->PridejKomponentu(DenVTydnu);
    TLabel * Den = new TLabel(0);
    _SeznamVytvorenychKomponent->PridejKomponentu(Den);
    TLabel * Mesic = new TLabel(0);
    _SeznamVytvorenychKomponent->PridejKomponentu(Mesic);

    DenVTydnu->Caption = D.FormatString("ddd");
    DenVTydnu->Parent = (TWinControl *) _Parent;
    DenVTydnu->Left = _lhrX;
    DenVTydnu->Top = _lhrY;
    _lhrY += DenVTydnu->Height;

    Den->Caption = D.FormatString("dd");
    Den->Parent = (TWinControl *) _Parent;
    Den->Left = _lhrX;
    Den->Top = _lhrY;
    _lhrY += Den->Height;

    Mesic->Caption = D.FormatString("mm");
    Mesic->Parent = (TWinControl *) _Parent;
    Mesic->Left = _lhrX;
    Mesic->Top = _lhrY;
    _lhrY += Mesic->Height;

    if(D.DayOfWeek() == 1) // nedle
    {
        DenVTydnu->Color = clRed;
        Den->Color = clRed;
        Mesic->Color = clRed;
    }
    return _lhrY;
}
//----------------------------------------------------------------------------

